@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2015 Andrew Ziem
-# http://bleachbit.sourceforge.net
+# Copyright (C) 2008-2019 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,6 +22,10 @@
 Check for updates via the Internet
 """
 
+from __future__ import absolute_import, print_function
+
+import bleachbit
+from bleachbit import _
 
 import hashlib
 import logging
@@ -30,18 +34,21 @@ import os.path
 import platform
 import socket
 import sys
-import traceback
-import urllib2
+if sys.version >= (3, 0):
+    from urllib.request import build_opener
+    from urllib.error import URLError
+else:
+    from urllib2 import build_opener, URLError
+
 import xml.dom.minidom
 
-import Common
-from Common import _
+logger = logging.getLogger(__name__)
 
 
 def update_winapp2(url, hash_expected, append_text, cb_success):
     """Download latest winapp2.ini file.  Hash is sha512 or None to disable checks"""
     # first, determine whether an update is necessary
-    from Common import personal_cleaners_dir
+    from bleachbit import personal_cleaners_dir
     fn = os.path.join(personal_cleaners_dir, 'winapp2.ini')
     delete_current = False
     if os.path.exists(fn):
@@ -53,13 +60,9 @@ def update_winapp2(url, hash_expected, append_text, cb_success):
         f.close()
         delete_current = True
     # download update
-    opener = urllib2.build_opener()
+    opener = build_opener()
     opener.addheaders = [('User-Agent', user_agent())]
-    kwargs = {'fullurl': url}
-    if sys.hexversion >= 0x02060000:
-        # Python 2.6 added timeout option
-        kwargs['timeout'] = 20
-    doc = opener.open(**kwargs).read()
+    doc = opener.open(fullurl=url, timeout=20).read()
     # verify hash
     hash_actual = hashlib.sha512(doc).hexdigest()
     if hash_expected and not hash_actual == hash_expected:
@@ -67,7 +70,7 @@ def update_winapp2(url, hash_expected, append_text, cb_success):
                            (url, hash_actual, hash_expected))
     # delete current
     if delete_current:
-        from FileUtilities import delete
+        from bleachbit.FileUtilities import delete
         delete(fn, True)
     # write file
     if not os.path.exists(personal_cleaners_dir):
@@ -102,7 +105,6 @@ def user_agent():
         import locale
         __locale = locale.getdefaultlocale()[0]  # e.g., en_US
     except:
-        logger = logging.getLogger(__name__)
         logger.exception('Exception when getting default locale')
 
     try:
@@ -111,7 +113,7 @@ def user_agent():
     except:
         gtkver = ""
 
-    agent = "BleachBit/%s (%s; %s; %s%s)" % (Common.APP_VERSION,
+    agent = "BleachBit/%s (%s; %s; %s%s)" % (bleachbit.APP_VERSION,
                                              __platform, __os, __locale, gtkver)
     return agent
 
@@ -119,7 +121,7 @@ def user_agent():
 def update_dialog(parent, updates):
     """Updates contains the version numbers and URLs"""
     import gtk
-    from GuiBasic import open_url
+    from bleachbit.GuiBasic import open_url
     dlg = gtk.Dialog(title=_("Update BleachBit"),
                      parent=parent,
                      flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
@@ -152,29 +154,28 @@ def update_dialog(parent, updates):
 
 def check_updates(check_beta, check_winapp2, append_text, cb_success):
     """Check for updates via the Internet"""
-    opener = urllib2.build_opener()
-    socket.setdefaulttimeout(Common.socket_timeout)
+    opener = build_opener()
+    socket.setdefaulttimeout(bleachbit.socket_timeout)
     opener.addheaders = [('User-Agent', user_agent())]
-    logger = logging.getLogger(__name__)
     try:
-        handle = opener.open(Common.update_check_url)
-    except urllib2.URLError:
+        handle = opener.open(bleachbit.update_check_url)
+    except URLError:
         logger.exception(
             _('Error when opening a network connection to %s to check for updates. Please verify the network is working.' %
-                Common.update_check_url))
+                bleachbit.update_check_url))
         return ()
     doc = handle.read()
     try:
         dom = xml.dom.minidom.parseString(doc)
     except:
-        logger.exception('The update information does not parse: %s' % doc)
+        logger.exception('The update information does not parse: %s', doc)
         return ()
 
     def parse_updates(element):
         if element:
             ver = element[0].getAttribute('ver')
             url = element[0].firstChild.data
-            return (ver, url)
+            return ver, url
         return ()
 
     stable = parse_updates(dom.getElementsByTagName("stable"))
@@ -189,9 +190,9 @@ def check_updates(check_beta, check_winapp2, append_text, cb_success):
     dom.unlink()
 
     if stable and beta and check_beta:
-        return (stable, beta)
+        return stable, beta
     if stable:
-        return (stable,)
+        return stable,
     if beta and check_beta:
-        return (beta,)
+        return beta,
     return ()
