@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2018 Andrew Ziem
+# Copyright (C) 2008-2019 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -390,18 +390,21 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
 
         # File is open with exclusive lock, so expect the file is neither
         # deleted nor truncated.
-        filename = test_delete_locked_setup()
-        from subprocess import call
-        call('start notepad.exe>>{}'.format(filename), shell=True)
-        self.assertExists(filename)
-        self.assertEqual(3, getsize(filename))
-        with self.assertRaises(WindowsError):
+        for allow_shred in (False, True):
+            filename = test_delete_locked_setup()
+            self.assertEqual(3, getsize(filename))
+            fd = os.open(filename, os.O_APPEND | os.O_EXCL)
+            self.assertExists(filename)
+            self.assertEqual(3, getsize(filename))
+            with self.assertRaises(WindowsError):
+                delete(filename, shred=allow_shred, allow_shred=allow_shred)
+            os.close(fd)
+            self.assertExists(filename)
+            if not allow_shred:
+                # A shredding attempt truncates the file.
+                self.assertEqual(3, getsize(filename))
             delete(filename)
-        call('taskkill /f /im notepad.exe')
-        self.assertExists(filename)
-        self.assertEqual(3, getsize(filename))
-        delete(filename)
-        self.assertNotExists(filename)
+            self.assertNotExists(filename)
 
     @unittest.skipIf('nt' == os.name, 'skipping on Windows')
     def test_ego_owner(self):
@@ -634,6 +637,27 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
                 this_drive = os.path.splitdrive(drive)[0]
                 self.assertEqual(same_partition(home, drive), home_drive == this_drive)
 
+    def test_uris_to_paths(self):
+        """Unit test for uris_to_paths()"""
+        self.assertEqual(uris_to_paths(['']), [])
+
+        # Unix-style
+        uri_u = ['file:///usr/bin/bleachbit']
+        path_u = [os.path.normpath(u'/usr/bin/bleachbit'), ]
+        self.assertEqual(uris_to_paths(uri_u), path_u)
+
+        # Windows
+        uri_w = [r'file:///C:/program%20files/bleachbit.exe']
+        path_w = [os.path.normpath(ur'C:/program files/bleachbit.exe'), ]
+        self.assertEqual(uris_to_paths(uri_w), path_w)
+
+        # Multiple
+        self.assertEqual(uris_to_paths(uri_u + uri_w), path_u + path_w)
+
+        # Unsupported scheme
+        uri_s = ['foo://bar']
+        self.assertEqual(uris_to_paths(uri_u + uri_w + uri_s), path_u + path_w)
+
     def test_whitelisted(self):
         """Unit test for whitelisted()"""
         # setup
