@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2018 Andrew Ziem
+# Copyright (C) 2008-2019 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -151,3 +151,18 @@ class OptionsTestCase(common.BleachbitTestCase):
 
         # clean up
         del o
+
+    def test_percent(self):
+        """Test that the percent sign can be used without quoting the string"""
+        # https://github.com/bleachbit/bleachbit/issues/205
+
+        opt = bleachbit.Options.options.config
+        if not opt.has_section('test'):
+            opt.add_section('test')
+        opt.set('test', 'filename', '/var/log/samba/log.%m')
+
+        # read
+        self.assertEqual(opt.get('test', 'filename'), '/var/log/samba/log.%m')
+
+        # clean up
+        del opt
