@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2018 Andrew Ziem
+# Copyright (C) 2008-2019 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -23,52 +23,53 @@ Test cases for module Winapp
 
 from __future__ import absolute_import, print_function
 
-from tests import common
-from bleachbit.Winapp import Winapp, detectos, detect_file, section2option
-from bleachbit.Windows import detect_registry_key, parse_windows_build
-from bleachbit import logger
-
 import os
 import shutil
 import sys
 import tempfile
 import unittest
 
+from tests import common
+from bleachbit.Winapp import Winapp, detectos, detect_file, section2option
+from bleachbit.Windows import detect_registry_key, parse_windows_build
+from bleachbit import logger
 
-def CreateSubKey(sub_key):
+
+def create_sub_key(sub_key):
+    """Create a registry key"""
     import _winreg
     hkey = _winreg.CreateKey(_winreg.HKEY_CURRENT_USER, sub_key)
     hkey.Close()
 
 
-keyfull = 'HKCU\\Software\\BleachBit\\DeleteThisKey'
+KEYFULL = 'HKCU\\Software\\BleachBit\\DeleteThisKey'
 
 
 def get_winapp2():
     """Download and cache winapp2.ini.  Return local filename."""
     url = "https://rawgit.com/bleachbit/winapp2.ini/master/Winapp2-BleachBit.ini"
     tmpdir = None
-    if 'posix' == os.name:
+    if os.name == 'posix':
         tmpdir = '/tmp'
-    if 'nt' == os.name:
+    if os.name == 'nt':
         tmpdir = os.getenv('TMP')
-    fn = os.path.join(tmpdir, 'bleachbit_test_winapp2.ini')
-    if os.path.exists(fn):
+    fname = os.path.join(tmpdir, 'bleachbit_test_winapp2.ini')
+    if os.path.exists(fname):
         import time
         import stat
-        age_seconds = time.time() - os.stat(fn)[stat.ST_MTIME]
+        age_seconds = time.time() - os.stat(fname)[stat.ST_MTIME]
         if age_seconds > (24 * 36 * 36):
-            logger.info('deleting stale file %s ', fn)
-            os.remove(fn)
-    if not os.path.exists(fn):
-        f = open(fn, 'w')
+            logger.info('deleting stale file %s ', fname)
+            os.remove(fname)
+    if not os.path.exists(fname):
+        fobj = open(fname, 'w')
         import urllib2
         txt = urllib2.urlopen(url).read()
-        f.write(txt)
-    return fn
+        fobj.write(txt)
+    return fname
 
 
-@unittest.skipUnless('win32' == sys.platform, 'not running on windows')
+@unittest.skipUnless(sys.platform == 'win32', 'not running on windows')
 class WinappTestCase(common.BleachbitTestCase):
     """Test cases for Winapp"""
 
@@ -109,14 +110,20 @@ class WinappTestCase(common.BleachbitTestCase):
                  ('6.2|', '6.0', False),
                  ('6.2|', '6.1', False),
                  ('6.2|', '6.2', True),
+                 # must be 6.2 or 6.3
+                 ('6.2|6.3', '6.0', False),
+                 ('6.2|6.3', '6.1', False),
+                 ('6.2|6.3', '6.2', True),
+                 ('6.2|6.3', '6.3', True),
+                 ('6.2|6.3', '10.0', False),
                  # 10.0 is the minimum
                  ('10.0|', '5.1', False),
                  ('10.0|', '10.0', True))
-        for (s, mock, expected_return) in tests:
+        for (req, mock, expected_return) in tests:
             mock = parse_windows_build(mock)
-            actual_return = detectos(s, mock)
+            actual_return = detectos(req, mock)
             self.assertEqual(expected_return, actual_return,
-                             'detectos(%s, %s)==%s instead of %s' % (s, mock,
+                             'detectos(%s, %s)==%s instead of %s' % (req, mock,
                                                                      actual_return, expected_return))
 
     def test_detect_file(self):
@@ -133,7 +140,7 @@ class WinappTestCase(common.BleachbitTestCase):
         # variable to also %ProgramW6432%, so test unique entries in
         # %ProgramW6432%.
         import struct
-        if not 32 == 8 * struct.calcsize('P'):
+        if 8 * struct.calcsize('P') != 32:
             raise NotImplementedError('expecting 32-bit Python')
         if os.getenv('ProgramW6432'):
             dir_64 = os.listdir(os.getenv('ProgramFiles'))
@@ -145,7 +152,8 @@ class WinappTestCase(common.BleachbitTestCase):
             for pathname in dir_32_unique:
                 tests.append(('%%ProgramFiles%%\\%s' % pathname, True))
         else:
-            logger.info('skipping %ProgramW6432% tests because WoW64 not detected')
+            logger.info(
+                'skipping %ProgramW6432% tests because WoW64 not detected')
         for (pathname, expected_return) in tests:
             actual_return = detect_file(pathname)
             msg = 'detect_file(%s) returned %s' % (pathname, actual_return)
@@ -155,28 +163,30 @@ class WinappTestCase(common.BleachbitTestCase):
         """Setup the test environment"""
         subkey = 'Software\\BleachBit\\DeleteThisKey\\AndThisKey'
 
-        dirname = tempfile.mkdtemp(prefix='bleachbit-test-winapp')
-        f1 = os.path.join(dirname, f1_filename or 'deleteme.log')
-        open(f1, 'w').close()
+        # put ampersand in directory name to test
+        # https://github.com/bleachbit/bleachbit/issues/308
+        dirname = tempfile.mkdtemp(prefix='bleachbit-test-winapp&')
+        fname1 = os.path.join(dirname, f1_filename or 'deleteme.log')
+        open(fname1, 'w').close()
 
         dirname2 = os.path.join(dirname, 'sub')
         os.mkdir(dirname2)
-        f2 = os.path.join(dirname2, 'deleteme.log')
-        open(f2, 'w').close()
+        fname2 = os.path.join(dirname2, 'deleteme.log')
+        open(fname2, 'w').close()
 
         fbak = os.path.join(dirname, 'deleteme.bak')
         open(fbak, 'w').close()
 
-        self.assertExists(f1)
-        self.assertExists(f2)
+        self.assertExists(fname1)
+        self.assertExists(fname2)
         self.assertExists(fbak)
 
-        CreateSubKey(subkey)
+        create_sub_key(subkey)
 
-        self.assertTrue(detect_registry_key(keyfull))
+        self.assertTrue(detect_registry_key(KEYFULL))
         self.assertTrue(detect_registry_key('HKCU\\%s' % subkey))
 
-        return dirname, f1, f2, fbak
+        return dirname, fname1, fname2, fbak
 
     def ini2cleaner(self, body, do_next=True):
         """Write a minimal Winapp2.ini"""
@@ -209,33 +219,32 @@ class WinappTestCase(common.BleachbitTestCase):
         # 4=filename1 (.\deleteme.log) exists after cleaning
         # 5=sub\deleteme.log exists after cleaning
         # 6=.\deleteme.bak exists after cleaning
-        # 7=auto-hide after cleaning
         tests = [
             # single file
             ('FileKey1=%s|deleteme.log', None,
-                False, True, False, True, True, True),
+                False, True, False, True, True),
             # single file, case matching should be insensitive
             ('FileKey1=%s|dEleteme.LOG', None,
-                False, True, False, True, True, True),
+                False, True, False, True, True),
             # special characters for XML
             ('FileKey1=%s|special_chars_&-\'.txt', 'special_chars_&-\'.txt',
-             False, True, False, True, True, True),
+             False, True, False, True, True),
             # *.log
-            ('FileKey1=%s|*.LOG', None, False, True, False, True, True, True),
+            ('FileKey1=%s|*.LOG', None, False, True, False, True, True),
             # semicolon separates different file types
             ('FileKey1=%s|*.log;*.bak', None,
-             False, True, False, True, False, True),
+             False, True, False, True, False),
             # *.*
-            ('FileKey1=%s|*.*', None, False, True, False, True, False, True),
+            ('FileKey1=%s|*.*', None, False, True, False, True, False),
             # recurse *.*
             ('FileKey1=%s|*.*|RECURSE', None, False,
-             True, False, False, False, True),
+             True, False, False, False),
             # recurse *.log
             ('FileKey1=%s|*.log|RECURSE', None, False,
-             True, False, False, True, True),
+             True, False, False, True),
             # remove self *.*, this removes the directory
             ('FileKey1=%s|*.*|REMOVESELF', None,
-             False, False, False, False, False, True),
+             False, False, False, False, False),
         ]
 
         # Add positive detection, where the detection believes the application is present,
@@ -244,13 +253,18 @@ class WinappTestCase(common.BleachbitTestCase):
         for test in tests:
             for detect in (
                 "\nDetectFile=%%APPDATA%%\\Microsoft",
+                "\nSpecialDetect=DET_WINDOWS",
                 "\nDetectFile1=%%APPDATA%%\\Microsoft\nDetectFile2=%%APPDATA%%\\does_not_exist",
                 "\nDetectFile1=%%APPDATA%%\\does_not_exist\nDetectFile2=%%APPDATA%%\\Microsoft",
                 "\nDetect=HKCU\\Software\\Microsoft",
                 # Below checks that a space is OK in the registry key
                 "\nDetect=HKCU\\Software\\Microsoft\\Command Processor",
+                # Below checks Detect# where one of two keys exist.
                 "\nDetect1=HKCU\\Software\\Microsoft\nDetect2=HKCU\\Software\\does_not_exist",
-                    "\nDetect1=HKCU\\Software\\does_not_exist\nDetect2=HKCU\\Software\\Microsoft"):
+                "\nDetect1=HKCU\\Software\\does_not_exist\nDetect2=HKCU\\Software\\Microsoft",
+                # Below checks Detect with DetectFile where one exists
+                "\nDetect=HKCU\\Software\\Microsoft\nDetectFile=%%APPDATA%%\\does_not_exist",
+                "\nDetect=HKCU\\Software\\does_not_exist\nDetectFile=%%APPDATA%%\\Microsoft"):
                 new_ini = test[0] + detect
                 new_test = [new_ini, ] + [x for x in test[1:]]
                 new_tests.append(new_test)
@@ -259,16 +273,16 @@ class WinappTestCase(common.BleachbitTestCase):
         # execute positive tests
         for test in positive_tests:
             print('positive test: ', test)
-            (dirname, f1, f2, fbak) = self.setup_fake(test[1])
+            self.assertEqual(len(test), 7)
+            (dirname, fname1, fname2, fbak) = self.setup_fake(test[1])
             cleaner = self.ini2cleaner(test[0] % dirname)
             self.assertEqual(test[2], cleaner.auto_hide())
             self.run_all(cleaner, False)
             self.run_all(cleaner, True)
             self.assertCondExists(test[3], dirname)
-            self.assertCondExists(test[4], f1)
-            self.assertCondExists(test[5], f2)
+            self.assertCondExists(test[4], fname1)
+            self.assertCondExists(test[5], fname2)
             self.assertCondExists(test[6], fbak)
-            self.assertEqual(test[7], cleaner.auto_hide())
             shutil.rmtree(dirname, True)
 
         # negative tests where the application detect believes the application
@@ -276,6 +290,7 @@ class WinappTestCase(common.BleachbitTestCase):
         for test in tests:
             for detect in (
                 "\nDetectFile=c:\\does_not_exist",
+                "\nSpecialDetect=DET_SPACE_QUEST",
                 # special characters for XML
                 "\nDetectFile=c:\\does_not_exist_special_chars_&'",
                 "\nDetectFile1=c:\\does_not_exist1\nDetectFile2=c:\\does_not_exist2",
@@ -283,25 +298,25 @@ class WinappTestCase(common.BleachbitTestCase):
                 "\nDetect=HKCU\\Software\\does_not_exist_&'",
                     "\nDetect1=HKCU\\Software\\does_not_exist1\nDetect2=HKCU\\Software\\does_not_exist1"):
                 new_ini = test[0] + detect
-                t = [new_ini, ] + [x for x in test[1:]]
-                print('negative test', t)
+                test_full = [new_ini, ] + [x for x in test[1:]]
+                print('negative test', test_full)
                 # execute the test
-                (dirname, f1, f2, fbak) = self.setup_fake()
-                cleaner = self.ini2cleaner(t[0] % dirname, False)
+                (dirname, fname1, fname2, fbak) = self.setup_fake()
+                cleaner = self.ini2cleaner(test_full[0] % dirname, False)
                 self.assertRaises(StopIteration, cleaner.next)
                 shutil.rmtree(dirname, True)
 
         # registry key, basic
-        (dirname, f1, f2, fbak) = self.setup_fake()
-        cleaner = self.ini2cleaner('RegKey1=%s' % keyfull)
+        (dirname, fname1, fname2, fbak) = self.setup_fake()
+        cleaner = self.ini2cleaner('RegKey1=%s' % KEYFULL)
         self.run_all(cleaner, False)
-        self.assertTrue(detect_registry_key(keyfull))
+        self.assertTrue(detect_registry_key(KEYFULL))
         self.run_all(cleaner, True)
-        self.assertFalse(detect_registry_key(keyfull))
+        self.assertFalse(detect_registry_key(KEYFULL))
         shutil.rmtree(dirname, True)
 
         # check for parse error with ampersand
-        (dirname, f1, f2, fbak) = self.setup_fake()
+        (dirname, fname1, fname2, fbak) = self.setup_fake()
         cleaner = self.ini2cleaner(
             'RegKey1=HKCU\\Software\\PeanutButter&Jelly')
         self.run_all(cleaner, False)
@@ -379,7 +394,7 @@ class WinappTestCase(common.BleachbitTestCase):
         for test in tests:
             msg = '\nTest:\n%s' % test[0]
             # setup
-            (dirname, f1, f2, fbak) = self.setup_fake()
+            (dirname, _fname1, _fname2, _fbak) = self.setup_fake()
             self.assertExists(r'%s\deleteme.log' % dirname, msg)
             self.assertExists(r'%s\deleteme.bak' % dirname, msg)
             self.assertExists(r'%s\sub\deleteme.log' % dirname, msg)
