@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2018 Andrew Ziem
+# Copyright (C) 2008-2019 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -134,6 +134,10 @@ def destructive_tests(title):
 
 def touch_file(filename):
     """Create an empty file"""
+    dname = os.path.dirname(filename)
+    if not os.path.exists(dname):
+        # Make the directory, if it does not exist.
+        os.makedirs(dname)
     with open(filename, "w") as f:
         pass
     assert(os.path.exists(filename))
